||----------------------------------------------------------------------|| 
||									||
||	pretty.m							||
||									||
||	Pretty printing program for the abstract syntax			||
||	used in denotational semantics.					||
||									||
||	(c) Simon Thompson, 1995					||
||									||
||----------------------------------------------------------------------|| 

%include "abstract_syntax"

||----------------------------------------------------------------------||
||									||
||	Printing is performed thus:					||
||									||
||	One command at most per line, control constructs are split	||
||	across lines in the usual way. Expressions are assumed to	||
||	fit one a single line. This seems to be the prctice for		||
||	Pascal, but could be changed easily enough. Definitions are	||
||	listed one per line, and are terminated by semi-colons. 	||
||									||
||	It is worth noting that because ';' is a separator for 		||
||	commands, we can only insert ';'s between commands explicitly	||
||	sequenced by 'Sequence', and not terminate the pretty		||
||	printing of each command with a ';'. This means in turn that 	||
||	we cannot terminate such a string with a newline, as we		||
||	may want to add a separating ';' at a higher level.		||
||									||
||----------------------------------------------------------------------||

||----------------------------------------------------------------------||
||									||
||	Pretty printing of commands					||
||									||
||	The main function is						||
||		pretty :: command -> [char]				||
||	but this uses the auxilliary					||
||		pre :: num -> command -> [char]				||
||	which carries the appropriate indentation info			||
||	in its first argument.						||
||									||
||----------------------------------------------------------------------||

pretty :: command -> [char]

pre :: num -> command -> [char]

pretty com = pre 0 com

pre n Skip = spaces n ++ "skip"
pre n (If_Then_Else e c1 c2) = spaces n ++ "if " ++ b_pretty e ++ "\n"
				++ spaces (n+2) ++ "then" ++ "\n"
				++ pre (n+2) c1 ++ "\n"
				++ spaces (n+2) ++ "else" ++ "\n"
				++ pre (n+2) c2 
pre n (While_Do b c) = spaces n ++ "while " ++ b_pretty b ++ " do" ++ "\n"
			++ pre (n+2) c
pre n (Repeat_Until c b) = spaces n ++ "repeat " ++ "\n"
			    ++ pre (n+2) c ++ "\n"
			    ++ "until " ++ b_pretty b
pre n (Sequence l) = spaces n ++ "begin" ++ "\n"
			++ pretty_list (pre (n+2)) ";\n" l ++ "\n"
			++ spaces n ++ "end" 
pre n (Assignment id e) = spaces n ++ i_pretty id ++ ":=" ++ e_pretty e


pretty_list :: (* -> [char]) -> [char] -> [*] -> [char]

||----------------------------------------------------------------------||
||	pretty_list prettifies every item in the list (3rd arg.)	||
||	using the function f, and flattens the result, separating	||
||	items with sep.							||
||----------------------------------------------------------------------||

pretty_list f sep [] = []
pretty_list f sep [a] = f a
pretty_list f sep (a:x) = (f a) ++ sep ++ pretty_list f sep x

||----------------------------------------------------------------------||
||	Pretty printing expressions.					||
||----------------------------------------------------------------------||

b_pretty (Bool tv) = show tv
b_pretty (Apply_rel r e1 e2) = "(" ++ e_pretty e1 ++ r_pretty r 
				++ e_pretty e2 ++ ")"
b_pretty (Apply_bop e e1 e2) = "(" ++ b_pretty e1 ++ bop_pretty e 
				++ b_pretty e2 ++ ")"
b_pretty (Apply_monbop e e1) = "(" ++ monbop_pretty e 
				++ b_pretty e1 ++ ")"

r_pretty Greater = ">"
r_pretty Greater_eq = ">="
r_pretty Equal = "="

bop_pretty And = " and "
bop_pretty Or  = " or "

monbop_pretty Not = "not "

i_pretty (Name x) = x

e_pretty (Var id) = i_pretty id
e_pretty (Num n) = show n
e_pretty (Apply_nop op e1 e2) = "("++ e_pretty e1 ++ nop_pretty op
				++ e_pretty e2 ++ ")"
e_pretty (Apply_monop op e) =  monop_pretty op ++ "(" ++ e_pretty e ++ ")"

nop_pretty Add = "+"
nop_pretty Subtract = "-"
nop_pretty Multiply = "*"

monop_pretty Abs = "abs"
monop_pretty Minus = "-"

