||----------------------------------------------------------------------|| 
||									||
||	examples.m							||
||									||
||	Example programs in the abstract syntax for the language	||
||	whose denotational semantics is specified in this directory.	||
||									||
||	(c) Simon Thompson, 1995					||
||									||
||----------------------------------------------------------------------|| 

%include "abstract_syntax"

||----------------------------------------------------------------------||
||									||
||	Example programs						||
||	are given by the expressions					||
||	example, example2 - example 4 and example 10			||
||									||
||----------------------------------------------------------------------||

com1 = Assignment a (Num 2)

a = Name "a"

com2 = Assignment b (Num 3)

b = Name "b"

com3 = While_Do (Apply_rel Greater (Var a) (Num 0)) com4

com4 = Sequence [ (Assignment b (Apply_nop Add (Var b) (Num 1))) ,
		  (Assignment a (Apply_nop Subtract (Var a) (Num 1))) ]

example = Sequence [com1,com2,com3]

||----------------------------------------------------------------------||
||									||
||	example is							||
||									||
||	a := 2;		{com1}						||
||	b := 3;		{com2}						||
||	while (a>0) do	{com3}						||
||		begin							||
||		b:=b+1; {com4}						||
||		a:=a-1							||
||		end							||
||									||
||	output prints the pair of values (a,b) on termination,		||
||	after running the program on the initially empty store, st0.	||
||	An example using parallel assignment:				||
||	a := 2;		{com1}						||
||	b := 3;		{com2}						||
||	a,b := b,a;							||
||	is given in example2, and its sequential analogue		||
||	a := 2;		{com1}						||
||	b := 3;		{com2}						||
||	a := b;								||
||	b := a;								||
||	in example3.							||
||									||
||----------------------------------------------------------------------||

example2 = Sequence [ com1 , com2 , Par_assignment [ (a,Var b) , (b, Var a) ] ]

example3 = Sequence [ com1 , com2 ,
  		Sequence [ Assignment a (Var b) ,Assignment b (Var a) ] ]

example4 = Sequence [ example , example , example3,
		(While_Do expo example3) , 
		(If_Then_Else expo2 example example3) ]

expo = Apply_bop And (Apply_rel Greater_eq
			(Apply_nop Multiply (Var a) (Var b))	
			(Apply_monop Minus (Num 37)))
		     (Bool False)

expo2 = expo

||----------------------------------------------------------------------||
||									||
||	A final example: Using procedure definition and call,		||
||	together with constant and variable declarations. The output	||
||	produced is:							||
||									||
||	begin block							||
||	  const pie = 3.139999 ;					||
||	  var eric ;							||
||	  var fred ;							||
||	  procedure jane(simon,clare) 					||
||	    begin							||
||	      simon:=(simon+pie);					||
||	      jane:=(eric*fred)						||
||	    end;							||
||	  jane(abs(pie),fred)						||
||	end block							||
||									||
||----------------------------------------------------------------------||

example10 = Block dlist1 comm1

[pie,eric,fred,jane,simon,clare]
 = [Name "pie",Name "eric",Name "fred", Name "jane",Name "simon",Name "clare"]

d1 = Const pie (Num 3.14)
d2 = Variable eric
d3 = Variable fred
d4 = Procedure jane plist body

dlist1 = [d1,d2,d3,d4]

plist = [simon,clare]

body = Sequence [c10,c11]

c10 = Assignment simon (Apply_nop Add (Var simon) (Con pie))
c11 = Assignment jane (Apply_nop Multiply (Var eric) (Var fred))

comm1 = Call jane [e1,e2]

e1 = Apply_monop Abs (Con pie)

e2 = Var fred

||----------------------------------------------------------------------||
||	Example program for ref_procedures				||
||----------------------------------------------------------------------||

[joan,derby,joe,diana,swop,local]
 = [Name "joan",Name "derby",Name "joe",Name "diana",Name "swop",Name "local"]

||	list of identifiers

pro_dec = Procedure_ref swop [joan,derby] pro_body

pro_body = Block [ Variable local ] pro_body_com

pro_body_com = Sequence
		[	Assignment local (Var joan) ,
			Assignment joan  (Var derby) ,
			Assignment derby  (Var local)	]

prog = Block	[	Variable joe ,
			Variable diana ,
			pro_dec		]

		prog_body

prog_body = Sequence
		[	Assignment joe (Num 2) ,
			Assignment diana (Num 3) ,
			Call_ref swop [ joe , diana ]	]

||----------------------------------------------------------------------||
||									||
||	A recursive procedure for producing factorial			||
||									||
||	can be pretty printed using pretty4				||
||									||
||----------------------------------------------------------------------||

[fac,ans,no,di,chas,loca]
	= [Name "fac",Name "ans",Name "no",Name "di",Name "chas",Name "loca"]

facto = Block [	Variable di ,
		Variable chas ,
		Procedure_rec fac [ans,no] pr_body ]
	      commo

pr_body = Block [ Variable loca ]
		(If_Then_Else (Apply_rel Greater_eq (Num 0) (Var no))
			(Assignment ans (Num 1))
			recursive_call)

recursive_call
	= Sequence [
	  (Assignment loca (Var no)),
	  (Assignment no (Apply_nop Subtract (Var no) (Num 1))) ,
	  Call_rec fac [ans,no] ,
	  (Assignment ans (Apply_nop Multiply (Var ans) (Var loca))) ]

commo = Sequence [
	(Assignment chas (Num 3)),
	Call_rec fac [di,chas]	]


||----------------------------------------------------------------------|| 
||	An example to distinguish between dynamic and static binding	||
||	of definitions, as given in the lectures.			||
||----------------------------------------------------------------------|| 

(pI,x,y,circum,use) = (Name "pI",Name "x",Name "y",Name "circum",Name "use")

command1 = Block
		[
			Const pI (Num 3.1) ,
			Variable x	,
			Procedure circum  [y] 
				(Assignment x express1) ,
			Procedure use []
				( Block	[ Const pI (Num 3.2) ]
					( Call circum [ (Num 1) ] ) )
		]
		( Call use [] )

express1 = (Apply_nop Multiply (Num 2)
			(Apply_nop Multiply (Con pI) (Var y)))
