||----------------------------------------------------------------------|| 
||									||
||	machine.m							||
||									||
||      (c) Simon Thompson, 1995                                        ||
||									||
||	The operation of the machine whose code is given in the		||
||	file "code.m". The machine is stack-based, with a store		||
||	taking the form described in the file				||
||		../basic/semantic_types					||
||	Objects in the stack are numbers, booleans and identifiers,	||
||	tagged in an appropriate way in the type definition `item'.	||
||									||
||----------------------------------------------------------------------|| 

%include "code"

%include "../full/abstract_syntax"
%include "../basic/semantic_types" { ident == ident
				   }
%insert "../basic/expr_values.ins"

stack == [item]

item ::= N num | B bool | L ident

||----------------------------------------------------------------------|| 
||	All the instructions, bar the jumps and halt, 			||
||	pass control to the next instruction in the instruction		||
||	sequence. We can therefore explain their behaviour simply by	||
||	explaining their effect on the stack and store. This is	the	||
||	purpose of the 							||
||		execute_one						||
||	function.							||
||----------------------------------------------------------------------|| 

execute_one :: ( m_code , stack , stores ) -> ( stack , stores )

execute_one ( Push_num n , sta , sto )
	= ( N n : sta , sto )

execute_one ( Do_nop f , N v2 : N v1 : sta , sto )
	= ( N (nop_value f v1 v2) : sta , sto )

execute_one ( Do_monop f ,  N v : sta , sto )
	= ( N (monop_value f v) : sta , sto )

execute_one ( Contents ide , sta , sto )
	= ( N (lookup ide sto) : sta , sto )

execute_one ( Push_bool b , sta , sto )
	= (  B b : sta , sto )

execute_one ( Do_rel f , N v2 : N v1 : sta , sto )
	= ( B (rel_value f v1 v2) : sta , sto )

execute_one ( Do_bop f , B v2 : B v1 : sta , sto )
	= ( B (bop_value f v1 v2) : sta , sto )

execute_one ( Do_monbop f , B v : sta , sto )
	= ( B (monbop_value f v) : sta , sto )

execute_one ( Lval ide , sta , sto )
	= ( L ide : sta , sto )

execute_one ( Pop , top : sta , sto )
	= ( sta , sto )

execute_one ( Assign , L ide : N n : sta , sto )
	= ( sta , update sto ide n )

execute_one ( Copy , v : sta , sto )
	= ( v : v : sta , sto )

execute_one ( Label l , sta , sto )
	= ( sta , sto )

||----------------------------------------------------------------------|| 
||	The function follow when given a label and a program (i.e. a	||
||	list of m_code instructions) will return the portion of the 	||
||	program following the label. In other words it performs the 	||
||	jump.								||
||----------------------------------------------------------------------|| 

follow :: label -> [m_code] -> [m_code] 

follow l ((Label l):y) = y 
follow l  (inst : y)   = follow l y 
follow l      []       = error ("Label " ++ show l ++ " not found.")

||----------------------------------------------------------------------|| 
||	Given a program (which needs to be held to resolve the jumps)	||
||	and an intermediate stage in execution, i.e. a triple of	||
||		code sequence (first instruction to be performed	||
||				lies at the top)			||
||		stack							||
||		store							||
||	returns a similar triple on termination. We terminate with	||
||	either an explicit Halt instruction or when the code is		||
||	exhausted. We jump on jumps, and we use execute_one (step)	||
||	followed by a recursive call in all other cases.		||
||----------------------------------------------------------------------|| 

execute ::  [m_code] -> ( [m_code] , stack , stores ) ->
            ( [m_code] , stack , stores ) 

execute pro ( (Goto l) : re , sta , sto )
	= execute pro ( re' , sta , sto )
	  where
	  re' = follow l pro 

execute pro ( (Gofalse l) : re , B b : sta , sto )
	= execute pro ( re' , sta , sto ) 			, if ~ b
	= execute pro ( re  , sta , sto )			, otherwise
	  where
	  re' = follow l pro

execute pro ( (Gotrue l) : re , B b : sta , sto )
	= execute pro ( re' , sta , sto ) 			, if b
	= execute pro ( re  , sta , sto )			, otherwise
	  where
	  re' = follow l pro 

execute pro ( Halt : re , sta , sto )
	= ( re , sta , sto )

execute pro ( ins : re , sta , sto )
	= execute pro ( re , sta' , sto' )
	  where
	  ( sta' , sto' ) = execute_one ( ins, sta , sto )

execute pro ( []  , sta , sto ) = ( [] , sta , sto )

