||----------------------------------------------------------------------|| 
||									||
||	abstract_syntax.m						||
||									||
||	(c) Simon Thompson, 1995					||
||									||
||----------------------------------------------------------------------|| 


||----------------------------------------------------------------------||
||									||
||	Abstract Syntax 						||
||									||
||	The major syntactic categories are				||
||		commands 		command				||
||		definitions 		def				||
||		expressions 		expr				||
||	and	boolean expressions	b_expr				||
||	with various subsiduary categories defined. Note that		||
||	we have to impose an extra layer of structure here - for	||
||	instance we can't say that `an identifier is an			||
||	expression' - it has to be tagged as such (by Var in this	||
||	case).								||
||									||
||----------------------------------------------------------------------||

command ::= Skip |
	    If_Then_Else b_expr command command |
	    While_Do b_expr command |
	    Repeat_Until command b_expr |
	    Sequence [command] |
 	    Assignment ident expr |
	    Par_assignment [ ( ident , expr ) ] |
	    If_Then b_expr command |
	    Block [def] command |
	    Call ident [expr] |
	    Call_ref ident [ident] |		
	    Call_rec ident [ident] |
	    Call_vr ident [ident] 		

def ::=	Const ident expr |
	Variable ident |
	Procedure ident [ident] command |
	Procedure_ref ident [ident] command |	
	Procedure_rec ident [ident] command |
	Procedure_vr ident [ident] command	

expr ::= Con ident |
	 Var ident |
	 Num num |
	 Apply_nop nop expr expr |
	 Apply_monop monop expr

b_expr ::= Bool bool |
	   Apply_rel rel expr expr |
	   Apply_bop bop b_expr b_expr |
	   Apply_monbop monbop b_expr

||----------------------------------------------------------------------||
||	Other syntactic categories are					||
||									||
||		nop		binary num(erical) operators		||
||		monop		unary num(erical) operators		||
||		rel		binary num(erical) relations		||
||		bop		binary bool(ean) operators		||
||		monbop		unary bool(ean) operators		||
||	and 	ident		identifiers				||
||----------------------------------------------------------------------||

nop ::= Add | Subtract | Multiply
monop ::= Abs | Minus
rel ::= Greater | Greater_eq | Equal
bop ::= And | Or 
monbop ::= Not
ident ::= Name [char]


