||----------------------------------------------------------------------||
||									||
||	origami.m							||
||									||
||	origami - the art of folding					||
||									||
||	(c) Simon Thompson, 1995					||
||									||
||----------------------------------------------------------------------||

||----------------------------------------------------------------------||
||									||
||	foldl' folds from the front (left) of a list to the end		||
||	In cases where the operator is of type * -> * -> * and		||
||	is associative and commutative we get the same result as	||
||	we would with foldr. We use this function to update an		||
||	environment by a number of declarations, starting from 		||
||	the left, in our denotational semantics.			||
||									||
||	foldl2 does the same, except that it works down two lists	||
||	at the same time. An error results if the lists fail to		||
||	have the same length. We can define foldl2 from foldl using	||
||	the functions zip and unzip, which take pairs of lists to	||
||	lists of pairs and vice versa (respectively).			||
||									||
||	foldl2 f start l m = foldl' (uncurry f) start (zip l m)		||
||									||
||	In a similar way we can define a function foldr2		||
||									||
||----------------------------------------------------------------------||

foldl' :: (* -> ** -> **) -> ** -> [*] -> **

foldl' f start [] = start
foldl' f start (a:x) = foldl' f (f a start) x

foldl2 :: (* -> ** -> *** -> ***) -> *** -> [*] -> [**] -> ***

foldl2 f start [] [] = start
foldl2 f start (a:x) (b:y) = foldl2 f (f a b start) x y
foldl2 f start x [] = error "foldl2"
foldl2 f start [] y = error "foldl2"

foldr2 :: (* -> ** -> *** -> ***) -> *** -> [*] -> [**] -> ***

||	fold a function into two lists simultaneously

foldr2 f start l m = foldr (uncurry f) start (myzip l m)

myzip :: [*] -> [**] -> [(*,**)]

||	(myzip a) pair of lists into list of pairs

myzip [] [] = []
myzip (a:x) (b:y) = (a,b) : myzip x y
myzip x [] = error "myzip"
myzip [] y = error "myzip"

unzip :: [(*,**)] -> ([*],[**])

||	(unzip a) list of pairs into pair of lists
unzip [] = ([],[])
unzip ((a,b):z) = (a:x,b:y)
		  where
		  (x,y) = unzip z

uncurry :: (* -> ** -> ***) -> (*,**) -> ***

||	Turns a curried function into a (partially) uncurried one

uncurry f (x,y) = f x y
