||----------------------------------------------------------------------|| 
||									||
||	semantic_types.m						||
||									||
||	The types underlying the semantic interpretation of the		||
||	simple imperative language whose syntax appears in		||
||		abstract_syntax.m					||
||									||
||	(c) Simon Thompson, 1995					||
||									||
||----------------------------------------------------------------------|| 

%free	{ ident :: type
	}

||----------------------------------------------------------------------||
||									||
||	The semantic types 						||
||									||
||	The type of values consists of three different sorts of data	||
||	First we have numbers, the basic storable value for the		||
||	language, then we have the type of locations, here treated	||
||	as numbers (quite a reasonable definition, if we consider	||
||	how addressing takes place), and finally we have the values	||
||	of procedures. Procedures are interpreted as functions,		||
||	which take a store and a list of actual parameter values	||
||	as input, and return a store on output. One advantage of	||
||	doing denotational semantics in Miranda is that we can 		||
||	represent these types directly in the language.			||
||									||
||	Note that ref_procedures have the same type as value pro-	||
||	cedures. The only difference is in the value which we pass	||
||	to such a procedure on invocation. In the former case it is	||
||	an l-value (a location) and in the latter an r-value, a num	||
||	in this case.							||
||	This is true also of rec_procedures, which are passed variables	||
||	by reference.							||
||									||
||----------------------------------------------------------------------||

||----------------------------------------------------------------------|| 
||									||
||	Stores and environments are also defined here. They can be	||
||	thought of as abstract types with a limited interface		||
||	to the rest of the semantics.					||
||									||
||----------------------------------------------------------------------|| 

values ::= Number num |
	   Loc locations |
	   Abstract proc |
	   Abstract_vr vrproc

locations == num

proc == ( stores , [values] ) -> stores

vrproc == ( stores , [values] ) -> ( stores , [values] )

update :: stores -> locations -> values -> stores
lookup :: locations -> stores -> values

find :: ident -> env -> values
bind :: ident -> values -> env -> env
free :: env -> locations		

||----------------------------------------------------------------------||
||	Implementation of the types					||
||									||
||	The types are represented by association lists, pairs of	||
||		( index , datum	)					||
||	objects. Numerous other implementations exist, of course.	||
||----------------------------------------------------------------------|| 

stores == [( locations , values )]			

update x i val = (i,val):x

lookup i [] = error "lookup"
lookup i ((j,val):x) = val 			, if i=j
		     = lookup i x		, otherwise


env == [( ident , values )]			

find i [] = error "find"
find i ((j,val):x) = val 			, if i=j
		   = find i x			, otherwise

bind i val en = (i,val):en

free l = max locs + 1		, if locs ~= []
       = 0			, otherwise
	 where
	 locs = [ b | (a,(Loc b)) <- l ] 
